<cfcomponent output="false" extends="SimplePlugin">
	
	<!--- remember how easy CF makes Java access? --->
	<cfset System = createObject("java", "java.lang.System") />
	<cfset Thread = createObject("java", "java.lang.Thread") />
	<cfset File = createObject("java", "java.io.File") />
	<cfset Array = createObject("java", "java.lang.reflect.Array") />

	<cfset this.lib_DIRECTORY = getDirectoryFromPath(getCurrentTemplatePath()) & "hibernate_lib" />
	<cfset this.GROOVY_DIRECTORY = getDirectoryFromPath(getCurrentTemplatePath()) & "hibernate_groovy" />

	<cffunction name="init" access="public" output="false" returntype="HibernatePlugin">
		<cfargument name="entityPath" type="string" required="false"
			hint="Initial value for the 'entityPath' property" />
		<cfset super.init() />
		<cfset variables.entityPath = "" />
		<cfif structKeyExists(arguments, "entityPath")>
			<cfset setEntityPath(entityPath) />
		</cfif>
		<cfreturn this />
	</cffunction>
	
	
	
	<cffunction name="setEntityPath" access="public" output="false" returntype="void">
		<cfargument name="entityPath" type="string" required="true" />
		<cfif structKeyExists(variables, "cfgroovy")>
			<cfset variables.cfgroovy.close() />
		</cfif>
		<cfset variables.entityPath = entityPath />
	</cffunction>
	


	<cffunction name="setAnnotatedClasses" access="public" output="false" returntype="void">
		<cfargument name="annotatedClasses" type="array" required="true" />
		<cfif structKeyExists(variables, "cfgroovy")>
			<cfset variables.cfgroovy.close() />
		</cfif>
		<cfset variables.annotatedClasses = annotatedClasses />
	</cffunction>

	

	<cffunction name="setMappedClasses" access="public" output="false" returntype="void">
		<cfargument name="mappedClasses" type="array" required="true" />
		<cfif structKeyExists(variables, "cfgroovy")>
			<cfset variables.cfgroovy.close() />
		</cfif>
		<cfset variables.mappedClasses = mappedClasses />
	</cffunction>

	
	
	<cffunction name="setColdFusionDsn" access="public" output="false" returntype="void">
		<cfargument name="coldFusionDsn" type="string" required="true" />
		<cfif structKeyExists(variables, "cfgroovy")>
			<cfset variables.cfgroovy.close() />
		</cfif>
		<cftry>
			<cfset variables.coldFusionDsn = createObject("java", "coldfusion.server.ServiceFactory").dataSourceService.getDatasource(coldFusionDsn) />
			<cfcatch type="any">
				<cfthrow type="UnsupportedOperationException"
					message="Could not find a valid DataSource from DSN '#coldFusionDsn#'."
					detail="This can be caused by misconfiguring the DSN, disallowing ServiceFactory access, or using a CFML runtime that doesn't expose valid javax.sql.DataSources." />
			</cfcatch>
		</cftry>
	</cffunction>
	
	

	<cffunction name="setHibernateDialect" access="public" output="false" returntype="void">
		<cfargument name="hibernateDialect" type="string" required="true" />
		<cfif structKeyExists(variables, "cfgroovy")>
			<cfset variables.cfgroovy.close() />
		</cfif>
		<cfset variables.hibernateDialect = hibernateDialect />
	</cffunction>



	<cffunction name="getSessionFactory" access="public" output="false" returntype="any"
		hint="I return the Hibernate SessionFactory instance.">
		<cfif NOT structKeyExists(variables, "sessionFactoryWrapper")>
			<cfset variables.cfgroovy.configure() />
		</cfif>
		<cfreturn variables.sessionFactoryWrapper.getSessionFactory() />
	</cffunction>

	
	
	<cffunction name="close" access="public" output="false" returntype="void"
		hint="Called by CFGroovy.close to perform any plugin-specific cleanup.">
		<cfif structKeyExists(variables, "sessionFactoryWrapper")>
			<cfset variables.sessionFactoryWrapper.close() />
		</cfif>
		<cfset structDelete(variables, "sessionFactoryWrapper") />
	</cffunction>
	
	
	
	<cffunction name="initializeRequest" access="public" output="false" returntype="void">
		<cfif NOT structKeyExists(variables, "sessionFactoryWrapper")>
			<cfset variables.cfgroovy.configure() />
		</cfif>
		<cfset variables.sessionFactoryWrapper.setPageContext(getPageContext()) />
	</cffunction>
	
	
	
	<cffunction name="getCurrentSession" access="public" output="false" returntype="any"
		hint="I retrieve the current Hibernate session, creating a new one if needed.">
		<cfif NOT structKeyExists(variables, "sessionFactoryWrapper")>
			<cfset variables.cfgroovy.configure() />
		</cfif>
		<cfset initializeRequest() />
		<cfreturn variables.sessionFactoryWrapper.getCurrentSession() />
	</cffunction>
	
	
	
	<cffunction name="openSession" access="public" output="false" returntype="any"
		hint="I create a new Hibernate session and return it, closing the current one if there is one">
		<cfif NOT structKeyExists(variables, "sessionFactoryWrapper")>
			<cfset variables.cfgroovy.configure() />
		</cfif>
		<cfset initializeRequest() />
		<cfreturn variables.sessionFactoryWrapper.openSession() />
	</cffunction>
	
	
	
	<cffunction name="closeSession" access="public" output="false" returntype="void"
		hint="I close the current Hibernate session, if there is one">
		<cfif NOT structKeyExists(variables, "sessionFactoryWrapper")>
			<cfset variables.cfgroovy.configure() />
		</cfif>
		<cfset initializeRequest() />
		<cfreturn variables.sessionFactoryWrapper.closeSession() />
	</cffunction>
	
	
	
	<cffunction name="getSessionForRequest" access="public" output="false" returntype="any"
		deprecated="true">
		<cfreturn getCurrentSession() />
	</cffunction>
	
	
	
	<cffunction name="closeRequest" access="public" output="false" returntype="void"
		deprecated="true">
		<cfset closeSession() />
	</cffunction>
	
	
	
	<cffunction name="updateRuntimePath" access="public"  returntype="string">
		<cfargument name="path" type="string" required="true" />
		<cfset var jars = "" />
		<cfset var i = "" />

		<!--- add the hibernate jars to the path --->
		<cfdirectory action="list"
			directory="#this.LIB_DIRECTORY#"
			name="jars"
			filter="*.jar" />
		<cfloop query="jars">
			<cfset path = listAppend(path, jars.directory & File.separator & jars.name) />
		</cfloop>

		<!--- add the entityPath to the path for non-groovy assets (e.g. hibernate.cfg.xml) --->
		<!--- prepend it so that it will override the compiled path --->
		<cfset path = listPrepend(path, variables.entityPath) />

		<cfreturn path />
	</cffunction>


	
	<cffunction name="updateSourcePath" access="public"  returntype="string">
		<cfargument name="path" type="string" required="true" />
		<cfreturn listAppend(listAppend(path, variables.entityPath), this.GROOVY_DIRECTORY) />
	</cffunction>


	
	<cffunction name="updateBinding" access="public" output="false" returntype="any"
		hint="Invoked by CFGroovy.getBinding to add any needed bindings to the supplied Binding.  It may also replace the supplied Binding with a new one, but this is almost never what you want.">
		<cfargument name="binding" type="any" required="true" />
		<cfif structKeyExists(variables, "sessionFactoryWrapper")>
			<cfset binding.setVariable("sessionFactory", variables.sessionFactoryWrapper) />
			<!--- don't use sessionFactoryWrapper, it's deprecated, use sessionFactory instead --->
			<cfset binding.setVariable("sessionFactoryWrapper", variables.sessionFactoryWrapper) />
		</cfif>
		<cfreturn binding />
	</cffunction>
	
	

	<cffunction name="configure" access="public" output="false" returntype="void">
		<cfset fileCopy(
			getDirectoryFromPath(getCurrentTemplatePath()) & "hibernate_default.properties",
			variables.cfgroovy.WORK_DIRECTORY & File.separator & "hibernate.properties"
		) />
		<cfset variables.cfgroovy.runScript("hibernate_bootstrap.groovy", variables) />
	</cffunction>

<cfoutput></cfoutput>
</cfcomponent>
