<cfcomponent output="false">

	<!--- remember how easy CF makes Java access? --->
	<cfset System = createObject("java", "java.lang.System") />
	<cfset Thread = createObject("java", "java.lang.Thread") />
	<cfset File = createObject("java", "java.io.File") />
	<cfset Array = createObject("java", "java.lang.reflect.Array") />
	<cfset urlClazz = createObject("java", "java.net.URL").init("http://barneyb.com/").getClass() />
	<cfset stringClazz = createObject("java", "java.lang.String").getClass() />

	<cfset this.ENGINE_ID = dateFormat(now(), "yyyymmdd") & "." & timeFormat(now(), "HHmmss") & "." & createUUID() />
	<cfset this.WORK_DIRECTORY = getDirectoryFromPath(getCurrentTemplatePath()) & "work" & File.separator & "cfgroovy." & this.ENGINE_ID />
	<cfset this.PARAMS_REQUEST_KEY = "__cfgroovy_request_attributes_cache_" & this.ENGINE_ID />
	
	<cffunction name="init" access="public" output="false" returntype="cfgroovy">
		<cfargument name="path" type="string" required="false"
			hint="Initial value for the 'path' property" />
		<cfset variables.initialized = false />
		<cfset variables.lastInitializedAt = createDate(1970, 1, 1) />
		<cfset variables.plugins = arrayNew(1) />
		<cfif structKeyExists(arguments, "path")>
			<cfset setPath(path) />
		</cfif>
		<cfreturn this />
	</cffunction>
	
	
	
	<cffunction name="setPath" access="public" output="false" returntype="void">
		<cfargument name="path" type="string" required="true"
			hint="A comma delimited set of filesystem paths to search for groovy scripts.  Each path may contain a tree of packages per the Groovy (and Java) package conventions." />
		<cflock name="cfgroovy.#this.ENGINE_ID#.setClosingProperty" type="exclusive" timeout="10" throwontimeout="true">
			<cfset close() />
			<cfset variables.path = path />
		</cflock>
	</cffunction>
	
	
	
	<cffunction name="setPlugins" access="public" output="false" returntype="void">
		<cfargument name="plugins" type="array" required="true" />
		<cfset var i = "" />
		<cfset removeAllPlugins() />
		<cfloop from="1" to="#arrayLen(plugins)#" index="i">
			<cfset addPlugin(plugins[i]) />
		</cfloop>
	</cffunction>
	
	
	
	<cffunction name="addPlugin" access="public" output="false" returntype="void">
		<cfargument name="plugin" type="any" required="true" />
		<cflock name="cfgroovy.#this.ENGINE_ID#.setClosingProperty" type="exclusive" timeout="10" throwontimeout="true">
			<cfset close() />
			<cfset plugin.setCfGroovy(this) />
			<cfset arrayAppend(variables.plugins, plugin) />
		</cflock>
	</cffunction>
	
	
	
	<cffunction name="removeAllPlugins" access="public" output="false" returntype="void">
		<cflock name="cfgroovy.#this.ENGINE_ID#.setClosingProperty" type="exclusive" timeout="10" throwontimeout="true">
			<cfset close() />
			<cfset variables.plugins = arrayNew(1) />
		</cflock>
	</cffunction>
	
	
	
	<cffunction name="getLastInitializedAt" access="package" output="false" returntype="date">
		<cfreturn variables.lastInitializedAt />
	</cffunction>
	
	
	
	<cffunction name="close" access="public" output="false" returntype="void"
		hint="I close the CF Groovy environment.  This cleans up memory and filesystem resources the instance uses, and should always be called before disposing of an instance.">
		<cfset var i = "" />
		<cflock name="cfgroovy.#this.ENGINE_ID#.closing" type="exclusive" timeout="1" throwontimeout="false">
			<cfloop from="1" to="#arrayLen(variables.plugins)#" index="i">
				<cfset variables.plugins[i].close() />
			</cfloop>
			<cfset structDelete(variables, "rootLoader", false) />
			<cfset structDelete(variables, "groovyLoader", false) />
			<cfif directoryExists(this.WORK_DIRECTORY)>
				<cfdirectory action="delete"
					directory="#this.WORK_DIRECTORY#"
					recurse="true" />
			</cfif>
			<cfset variables.initialized = false />
		</cflock>
	</cffunction>



	<cffunction name="reloadIfNeeded" access="public" output="false" returntype="boolean"
		hint="I do a scan for changes in CF Groovy-managed assets and reload the runtime if needed to pick up any changes.  This should only be used in development as it the file I/O is expensive.">
		<cfif isReloadRequired()>
			<cfset close() />
			<cfset configure() />
			<cfreturn true />
		</cfif>
		<cfreturn false />
	</cffunction>


	
	<cffunction name="isReloadRequired" access="public" output="false" returntype="boolean"
		hint="Returns whether this instance needs to be closed and reconfigured.">
		<cfset var i = "" />
		<cfloop from="1" to="#arrayLen(variables.plugins)#" index="i">
			<cfif variables.plugins[i].isReloadRequired()>
				<cfreturn true />
			</cfif>
		</cfloop>
		<cfloop list="#variables.sourcePath#" index="i">
			<cfdirectory action="list"
				directory="#i#"
				name="fileList"
				recurse="true" />
			<cfquery dbtype="query" name="fileList" maxrows="1">
				select dateLastModified
				from fileList
				order by dateLastModified desc
			</cfquery>
			<cfif fileList.dateLastModified GT variables.lastInitializedAt>
				<cfreturn true />
			</cfif>
		</cfloop>
		<cfreturn false />
	</cffunction>
	
	

	<cffunction name="configure" access="public" output="false" returntype="void"
		hint="This method never needs to be invoked manually for the core of CF Groovy.  It will ALWAYS be invoked before any scripts are executed.  It may be necessary to invoke this method manually to ensure plugins' internal state is initialized for pre-script manipulations or to pay the initial hit at a known time rather than first request.">
		<cfset var urlArray = "" />
		<cfset var i = "" />
		<cfset var timestamp = "" />
		<cfset var doCompile = false />
		<cfif variables.initialized>
			<cfreturn />
		</cfif>
		<cftry>
			<cfset timestamp = now() />
			<cflock name="cfgroovy.#this.ENGINE_ID#.initializing" type="exclusive" timeout="30" throwontimeout="true">
				<cfif variables.initialized>
					<cfreturn />
				</cfif>
				<!--- this MUST be first --->
				<cfif NOT structKeyExists(variables, "path")>
					<cfset setPath("") />
				</cfif>
				
				<cfset variables.scriptClassCache = createObject("java", "java.util.HashMap").init() />
				<cfset variables.sourcePath = variables.path />
				<cfset variables.runtimePath = this.WORK_DIRECTORY />
				
				<!--- give the plugins a chance to influence the path --->
				<cfloop from="1" to="#arrayLen(variables.plugins)#" index="i">
					<cfset variables.sourcePath = variables.plugins[i].updateSourcePath(variables.sourcePath) />
					<cfset variables.runtimePath = variables.plugins[i].updateRuntimePath(variables.runtimePath) />
				</cfloop>
				<cfset doCompile = len(trim(variables.sourcePath)) GT 0 />
				
				<cfif doCompile AND NOT directoryExists(this.WORK_DIRECTORY)>
					<cfdirectory action="create"
						directory="#this.WORK_DIRECTORY#" />
				</cfif>
				
				<cfset urlArray = convertPathToUrlArray(variables.runtimePath) />

				<cfif doCompile>
					<cfset compileScripts(variables.sourcePath, this.WORK_DIRECTORY, urlArray) />
				</cfif>
	
				<!--- construct the loader --->
				<cfset variables.rootLoader = createObject("java", "org.codehaus.groovy.tools.RootLoader").init(
					urlArray, Thread.currentThread().getContextClassLoader()
				) />
				<cfset variables.groovyLoader = createObject("java", "groovy.lang.GroovyClassLoader").init(
					variables.rootLoader
				) />
				
				<cfset variables.initialized = true />
				<cfset variables.lastInitializedAt = timestamp />
	
				<cfloop from="1" to="#arrayLen(variables.plugins)#" index="i">
					<cfset variables.plugins[i].configure() />
				</cfloop>
			</cflock>
			<cfcatch type="any">
				<cfset close() />
				<cfrethrow />
			</cfcatch>
		</cftry>
	</cffunction>
	
	
	
	<cffunction name="compileScripts" access="package" output="false" returntype="void">
		<cfargument name="sourcePaths" type="string" required="true"
			hint="A comma-delimited list of paths to compile" />
		<cfargument name="outputDir" type="string" required="true" />
		<cfargument name="classPathUrlArray" type="any" required="true"
			hint="The compilation classpath as a URL[]." />
		<cfset var oldCCL = Thread.currentThread().getContextClassLoader() />
		<cfset var i = "" />
		<cfset var fileList = "" />
		<cfset var completeFileList = "" />
		<cfset var config = "" />
		<cfset var compiler = "" />
		<cfset var p = "" />

		<!--- build the list --->
		<cfloop list="#sourcePaths#" index="i">
			<cfif directoryExists(i)>
				<cfdirectory action="list"
					directory="#i#"
					name="fileList"
					filter="*.groovy"
					recurse="true" />
				<cfif isSimpleValue(completeFileList)>
					<cfset completeFileList = fileList />
				<cfelse>
					<cfquery dbtype="query" name="completeFileList">
							select *
							from completeFileList
						union
							select *
							from fileList
					</cfquery>
				</cfif>
			</cfif>
		</cfloop>
		
		<!--- compile the entities into the bin folder --->
		<cftry>
			<cfset Thread.currentThread().setContextClassLoader(createObject("java", "java.net.URLClassLoader").init(classPathUrlArray, Thread.currentThread().getContextClassLoader())) />
			<cfset config = createObject("java", "org.codehaus.groovy.control.CompilerConfiguration").init() />
			<cfset config.setTargetDirectory(outputDir) />
			<cfset compiler = createObject("java", "org.codehaus.groovy.tools.FileSystemCompiler").init(config) />
			<cfset stringArray = Array.newInstance(stringClazz, completeFileList.recordCount) />
			<cfloop query="completeFileList">
				<cfset p = directory & "/" & name />
				<cfset p = REReplace(p, "[\\/]+", File.separator, "all") />
				<cfset Array.set(stringArray, currentRow - 1, p)  />
			</cfloop>
			<cfset compiler.compile(stringArray) />
			<!--- where's my CFFINALLY???? --->
			<cfcatch type="any">
				<cfset Thread.currentThread().setContextClassLoader(oldCCL) />
				<cfrethrow />
			</cfcatch>
		</cftry>
		<cfset Thread.currentThread().setContextClassLoader(oldCCL) />
	</cffunction>
	
	
	
	<cffunction name="convertPathToUrlArray" access="package" output="false" returntype="any">
		<cfargument name="path" type="string" required="true" />
		<cfset var i = 0 />
		<cfset var urlArray = Array.newInstance(urlClazz, listLen(arguments.path)) />
		<cfset var item = "" />
		<cfloop list="#arguments.path#" index="item">
			<cfset Array.set(urlArray, i, createObject("java", "java.io.File").init(item).toURL()) />
			<cfset i = i + 1 />
		</cfloop>
		<cfreturn urlArray />
	</cffunction>
	


	<cffunction name="getBinding" access="private" output="false" returntype="any"
		hint="I build and return the Binding for the GroovyScriptEngine.">
		<cfargument name="variablesScope" type="struct" default="#structNew()#"
			hint="The variables scope to place in the 'variables' implicit variable.  Needn't be the actual variables scope of the caller, but usually will be." />
		<cfset var binding = createObject("java", "groovy.lang.Binding").init() />

		<cfif NOT structKeyExists(request, this.PARAMS_REQUEST_KEY)>
			<cfset request[this.PARAMS_REQUEST_KEY] = structNew() />
			<cfif isDefined("url")>
				<cfset structAppend(request[this.PARAMS_REQUEST_KEY], url) />
			</cfif>
			<cfif isDefined("form")>
				<cfset structAppend(request[this.PARAMS_REQUEST_KEY], form) />
			</cfif>
		</cfif>

		<cfset binding.setVariable("request", request) />
		<cfset binding.setVariable("cgi", cgi) />
		<cfset binding.setVariable("params", request[this.PARAMS_REQUEST_KEY]) />
		<!--- don't use attributes, it's deprecated, use params instead --->
		<cfset binding.setVariable("attributes", request[this.PARAMS_REQUEST_KEY]) />
		<cfset binding.setVariable("variables", variablesScope) />
		<cfset binding.setVariable("pageContext", getPageContext()) />
		<cfif isDefined("session")>
			<cfset binding.setVariable("session", session) />
		</cfif>
		<cfif isDefined("application")>
			<cfset binding.setVariable("application", application) />
		</cfif>
		<cfset binding.setVariable("server", server) />
			
		<!--- give the plugins a chance to influence the Binding --->
		<cfloop from="1" to="#arrayLen(variables.plugins)#" index="i">
			<cfset binding = variables.plugins[i].updateBinding(binding) />
		</cfloop>
		
		<cfreturn binding />
	</cffunction>

	
	
	<cffunction name="runScript" access="public" output="false" returntype="void"
		hint="Runs a named script from the configured path.">
		<cfargument name="scriptName" type="string" required="true"
			hint="The script to execute, including the '.groovy' extension." />
		<cfargument name="variablesScope" type="struct" default="#structNew()#"
			hint="The variables scope to place in the 'variables' implicit variable.  Needn't be the actual variables scope of the caller, but usually will be." />
		<cfset var script = "" />
		<!--- this will be a no-op in most cases --->
		<cfset configure() />
		<cfif scriptName.endsWith(".groovy")>
			<cfset scriptName = left(scriptName, len(scriptName) - 7) />
		</cfif>
		<cfset script = variables.rootLoader.loadClass(scriptName).newInstance() />
		<cfset script.setBinding(getBinding(variablesScope)) />
		<cfset script.run() />
	</cffunction>



	<cffunction name="runScriptBody" access="public" output="false" returntype="void"
		hint="Executes the script body.">
		<cfargument name="scriptContent" type="string" required="true"
			hint="A string Groovy code to be executed as a script" />
		<cfargument name="variablesScope" type="struct" default="#structNew()#"
			hint="The variables scope to place in the 'variables' implicit variable.  Needn't be the actual variables scope of the caller, but usually will be." />
		<cfargument name="filename" type="string" required="false"
			hint="The psuedo-filename to use for the script" />
		<cfset var cacheKey = "" />
		<cfset var l = structNew() />
		<cfset var script = "" />
		<cfif structKeyExists(arguments, "filename")>
			<cfset cacheKey = filename & ":" & chr(10) & scriptContent />
		<cfelse>
			<cfset cacheKey = scriptContent />
		</cfif>
		<!--- this will be a no-op in most cases --->
		<cfset configure() />
		<!--- this mumbo-jumbo caches parsed classes to avoid PermGen errors --->
		<cfset l.clazz = variables.scriptClassCache.get(cacheKey) />
		<cfif NOT structKeyExists(l, "clazz")>
			<cfif structKeyExists(arguments, "filename")>
				<cfset l.clazz = variables.groovyLoader.parseClass(scriptContent, filename) />
			<cfelse>
				<cfset l.clazz = variables.groovyLoader.parseClass(scriptContent) />
			</cfif>
			<cfset variables.scriptClassCache.put(cacheKey, l.clazz) />
		</cfif>
		<!--- now we'll actually create an instance of the script and run it --->
		<cfset script = l.clazz.newInstance() />
		<cfset script.setBinding(getBinding(variablesScope)) />
		<cfset script.run() />
	</cffunction>

<cfoutput></cfoutput>
</cfcomponent>
